/* 
 * Copyright (C) 2004 Romain Dolbeau <romain@dolbeau.org>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

package lavcGenetic;

import java.util.*;
import javax.swing.*;
import java.awt.event.*;
import java.io.*;

class lavcGeneticMainDisplay extends JPanel {
    private final JFrame parent;
    private final JLabel ccLabel;
    private final List allChooser = new ArrayList();
    private final Box stBox = Box.createHorizontalBox();
    private final Box rbBox = Box.createHorizontalBox();
    private final Box btBox = Box.createHorizontalBox();
    private final Map codecMap;
    private final Box mainBox = Box.createVerticalBox();
    private final Box codecBox = Box.createHorizontalBox();
    private final Box chooserBox = Box.createVerticalBox();
    private final Box bBox = Box.createHorizontalBox();

    private String currentCodec;
    private int nSimultaneousTests = lavcGeneticMain.kSimultaneousTests;
    private int nRandomBreeding;
    private int bitrate = lavcGeneticMain.kDefaultBitrate;
    private boolean twopass = true;
    
    class lavcSimultaneousTestsSliderListener implements javax.swing.event.ChangeListener {
        public void stateChanged(javax.swing.event.ChangeEvent e) {
            JSlider source = (JSlider)e.getSource();
            if (!source.getValueIsAdjusting()) {
                nSimultaneousTests = (int)source.getValue();
            }    
        }
    }

    class lavcRandomBreedingSliderListener implements javax.swing.event.ChangeListener {
        public void stateChanged(javax.swing.event.ChangeEvent e) {
            JSlider source = (JSlider)e.getSource();
            if (!source.getValueIsAdjusting()) {
                nRandomBreeding = (int)source.getValue();
            }    
        }
    }

    class lavcBitrateSliderListener implements javax.swing.event.ChangeListener {
        public void stateChanged(javax.swing.event.ChangeEvent e) {
            JSlider source = (JSlider)e.getSource();
            if (!source.getValueIsAdjusting()) {
                bitrate = (int)source.getValue();
            }    
        }
    }

    void addAllOptions() {
        chooserBox.removeAll();
        allChooser.clear();

        Set allOptions = ((lavcCodecData)codecMap.get(currentCodec)).allOptions;
        Iterator it = allOptions.iterator();
        while (it.hasNext()) {
            lavcOption lo = (lavcOption)it.next();
            lavcOptionChooser loc = new lavcOptionChooser(lo, this);
            if (lo.getAllVariants().size() > 1)
                chooserBox.add(loc);
            allChooser.add(loc);
        }
    }

    lavcGeneticMainDisplay(JFrame m, Map cm, String defaultCodec) {
        parent = m;
        parent.setVisible(false);
        codecMap = cm;
        currentCodec = defaultCodec;

        add(mainBox);

        mainBox.add(stBox);
        stBox.add(new JLabel("Initial number of run: "));
        JSlider stSlider = new JSlider(3, 300, 15);
        stSlider.setSnapToTicks(true);
        stSlider.setMinorTickSpacing(3);
        stSlider.setMajorTickSpacing(15);
        stSlider.setPaintTicks(true);
        stSlider.setPaintLabels(true);
        stSlider.addChangeListener(new lavcSimultaneousTestsSliderListener());
        stBox.add(stSlider);

        mainBox.add(rbBox);
        rbBox.add(new JLabel("Breeding Randomness: "));
        JSlider rbSlider = new JSlider(0, 100, 10);
        rbSlider.setSnapToTicks(false);
        rbSlider.setMinorTickSpacing(5);
        rbSlider.setMajorTickSpacing(25);
        rbSlider.setPaintTicks(true);
        rbSlider.setPaintLabels(true);
        rbSlider.addChangeListener(new lavcRandomBreedingSliderListener());
        rbBox.add(rbSlider);

        mainBox.add(btBox);
        btBox.add(new JLabel("Bitrate: "));
        JSlider btSlider = new JSlider(100, 1800, 650);
        btSlider.setSnapToTicks(false);
        btSlider.setMinorTickSpacing(50);
        btSlider.setMajorTickSpacing(250);
        btSlider.setPaintTicks(true);
        btSlider.setPaintLabels(true);
        btSlider.addChangeListener(new lavcBitrateSliderListener());
        btBox.add(btSlider);

        ButtonGroup codecGroup = new ButtonGroup();
        mainBox.add(codecBox);
        Iterator it = codecMap.keySet().iterator();
        while (it.hasNext()) {
            String codec = (String)it.next();
            JRadioButton jrb = new JRadioButton(new AbstractAction(codec) {
                    public void actionPerformed(ActionEvent e) {
                        currentCodec = e.getActionCommand();
                        addAllOptions();
                        refreshCCLabel();
                    }
                });
            if (codec.equals(currentCodec))
                jrb.setSelected(true);
            codecGroup.add(jrb);
            codecBox.add(jrb);
        }

        chooserBox.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
        mainBox.add(chooserBox);
        addAllOptions();
        

        JCheckBox jcb = new JCheckBox(new AbstractAction("Use 2-Pass Encoding ") {
                public void actionPerformed(ActionEvent e) {
                    JCheckBox jcb = (JCheckBox)e.getSource();
                    twopass = jcb.isSelected();
                }
            });
        jcb.setSelected(true);

        mainBox.add(bBox);
        ccLabel = new JLabel();
        bBox.add(jcb);
        bBox.add(ccLabel);
        bBox.add(new JButton(new AbstractAction("Explore Now !") {
                public void actionPerformed(ActionEvent e) {
                    cleanAndStart();
                }
            }));

        refreshCCLabel();
        
        parent.pack();
        parent.setVisible(true);
    }

    void cleanAndStart() {
        mainBox.remove(codecBox);
        
        chooserBox.removeAll();
        mainBox.remove(stBox);
        mainBox.remove(rbBox);
        mainBox.remove(btBox);
        mainBox.remove(bBox);
        
        final Set pos = getProcessedOptionsSet();
        final JTextArea jta = new JTextArea(pos.toString());
        jta.setLineWrap(true);
        jta.setWrapStyleWord(true);
        jta.setEditable(false);
        mainBox.add(jta, 0);
        
        mainBox.add(new JLabel("Codec: " + currentCodec), 0);
        mainBox.add(new JButton(new AbstractAction("stop (after step is done") {
                public void actionPerformed(ActionEvent e) {
                    lavcGeneticMain.setDone();
                };
            }));
        
        final lavcAllStatisticsDisplay lasd = new lavcAllStatisticsDisplay(pos, parent);
        mainBox.add(lasd);
        parent.pack();
        final Thread t = new Thread() {
                public void run() {
                    lavcGeneticMain.doTheJob(pos,
                                             lasd,
                                             (lavcCodecData)codecMap.get(currentCodec),
                                             nSimultaneousTests,
                                             nRandomBreeding,
                                             bitrate,
                                             twopass);
                }
            };
        t.start();
    }
    
    void refreshCCLabel() {
        int cc = lavcGeneticMain.numberOfCombinations(getProcessedOptionsSet());
        ccLabel.setText("Combinations selected: " + cc + " ");
        parent.pack();
    }

    private Set getProcessedOptionsSet() {
        Set temp = new HashSet();
        Iterator it = allChooser.iterator();
        while (it.hasNext()) {
            temp.add(((lavcOptionChooser)it.next()).getProcessedOption());
        }
        return temp;
    }
}
