/* 
 * Copyright (C) 2004 Romain Dolbeau <romain@dolbeau.org>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

package lavcGenetic;

import java.util.*;
import javax.swing.*;
import java.awt.event.*;

class lavcOptionChooser extends JPanel {
    private static final String kTryAll = "(try all)";
    private static final String kUnset = "(not set)";
    private final lavcOption myOption;
    private final ButtonGroup bg = new ButtonGroup();
    private final lavcGeneticMainDisplay parent;

    private int index = -1;

    void setIndex(int i) {
        index = i;
        System.err.println("Setting " + myOption.getDescription() + " to " + i);
    }

    class lavcOptionChooserButtonAction extends AbstractAction {
        lavcOptionChooserButtonAction(String name) {
            super(name);
        }
        public void actionPerformed(ActionEvent e) {
            String cmd = e.getActionCommand();
            if (cmd.equals(kTryAll))
                setIndex(-1);
            ListIterator it = myOption.getAllVariants().listIterator();
            while (it.hasNext()) {
                String name = ((lavcOptionInstance)it.next()).toString();
                if ((cmd.equals(name)) ||
                    (cmd.equals(kUnset) && (name == null)))
                    setIndex(it.previousIndex());
            }
            parent.refreshCCLabel();
        }
    }
    
    lavcOptionChooser(lavcOption m, lavcGeneticMainDisplay p) {
        myOption = m;
        parent = p;

        JRadioButton jrb;
        Box localBox = new Box(javax.swing.BoxLayout.X_AXIS);
        localBox.setAlignmentY(java.awt.Component.CENTER_ALIGNMENT);
        add(localBox);

        localBox.add(new JLabel(m.getDescription() + ": "));

        jrb = new JRadioButton(new lavcOptionChooserButtonAction(kTryAll));
        localBox.add(jrb);
        bg.add(jrb);
        jrb.setSelected(true);

        Iterator it = myOption.getAllVariants().iterator();
        while (it.hasNext()) {
            lavcOptionInstance loi = (lavcOptionInstance)it.next();
            String name = loi.toString();
            jrb = new JRadioButton(new lavcOptionChooserButtonAction(name == null ? kUnset : name));
            localBox.add(jrb);
            bg.add(jrb);
        }
    }
    
    lavcOption getProcessedOption() {
        if (index == -1)
            return myOption;

        String fixed = ((lavcOptionInstance)myOption.getAllVariants().get(index)).toString();

        return new lavcOption_Flag(fixed, true);
    }
}
