/* 
 * Copyright (C) 2004 Romain Dolbeau <romain@dolbeau.org>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

/* bad classname. it should be lavcOptionInstanceList, really */

package lavcGenetic;

import java.util.*;

class lavcOptionList extends ArrayList implements Comparable {
    
    lavcOptionList() {
        super();
    }
    
    lavcOptionList(List l) {
        super(l);
    }

    public int compareTo(Object o) {
        lavcOptionList o2 = (lavcOptionList)o;

        if (size() < o2.size())
            return -1;
        if (size() > o2.size())
            return 1;
        
        Iterator it1 = iterator();
        Iterator it2 = o2.iterator();
        while (it1.hasNext() && it2.hasNext()) {
            lavcOptionInstance i1 = (lavcOptionInstance)it1.next();
            lavcOptionInstance i2 = (lavcOptionInstance)it2.next();
            int c= i1.compareTo(i2);
            if (c != 0)
                return c;
        }
        return 0;
    }

    lavcOptionInstance getOptionInstance(lavcOption opt) {
        List allInstances = opt.getAllVariants();
        Iterator it = allInstances.iterator();
        while (it.hasNext()) {
            lavcOptionInstance loi = (lavcOptionInstance)it.next();
            if (contains(loi))
                return loi;
        }
        return null; // oups
    }

    static lavcOptionList buildRandomList(Set allOptions, Random random) {
        lavcOptionList temp = new lavcOptionList();
        Iterator it = allOptions.iterator();
        while (it.hasNext()) {
            lavcOption opt = (lavcOption)it.next();
            List allInstances = opt.getAllVariants();
            lavcOptionInstance loi = (lavcOptionInstance)allInstances.get(random.nextInt(allInstances.size()));
            temp.add(loi);
        }
        Collections.sort(temp);
        return temp;
    }
    
    void changeOneOptionAtRandom(Set allOptions, Random random) {
        lavcOption opt;
        List allInstances;
        int i = 0;

        do {
            opt = (lavcOption)allOptions.toArray()[random.nextInt(allOptions.size())];
            allInstances = opt.getAllVariants();
            i++;
        } while ((allInstances.size() < 2) && (i <= lavcGeneticMain.kMaxAttempts));

        if (allInstances.size() > 1) {
            lavcOptionInstance loi;
            int j = 0;
            do {
                loi= (lavcOptionInstance)allInstances.get(random.nextInt(allInstances.size()));
                j++;
            } while (contains(loi) && (j <= lavcGeneticMain.kMaxAttempts));

            if (!contains(loi)) {
                removeAll(allInstances);
                add(loi);
                Collections.sort(this);
            }
        }
    }

    static lavcOptionList breed(lavcOptionList l1, lavcOptionList l2, int oddsR, Set allOptions, Random random) {
        lavcOptionList temp = new lavcOptionList();
        Iterator it = allOptions.iterator();
        while (it.hasNext()) {
            lavcOption opt = (lavcOption)it.next();
            if (random.nextInt(100) < oddsR) {
                List allInstances = opt.getAllVariants();
                temp.add(allInstances.get(random.nextInt(allInstances.size())));
            } else {
                if (random.nextInt(100) < 50)
                    temp.add(l1.getOptionInstance(opt));
                else 
                    temp.add(l2.getOptionInstance(opt));
            }
        }
        Collections.sort(temp);
        return temp;
    }

    static List breedThemAll(List allButN, int max, lavcResultsPool lrp, int oddsR, Set allOptions, Random random) {
        List children = new ArrayList();
        for (int i = 0; (i < max) && (allButN.size() > 0) ; i++) {
            int j = 0;
            lavcOptionList c = null;
            do {
                lavcOptionList l1 = null;
                lavcOptionList l2 = null;
                int n1 = -1;
                int n2 = -1;
                n1 = random.nextInt(allButN.size());
                n2 = random.nextInt(allButN.size());
                l1 = (lavcOptionList)allButN.get(n1);
                l2 = (lavcOptionList)allButN.get(n2);
                c = breed(l1, l2, oddsR, allOptions, random);
                j++;
            } while ((children.contains(c) || lrp.hasResult(c)) && (j < lavcGeneticMain.kMaxAttempts));
            if (!(children.contains(c) || lrp.hasResult(c)))
                children.add(c);
        }
        return children;
    }

    public String toPrettyString() {
        String temp = null;
        Iterator it = iterator();
        while (it.hasNext()) {
            lavcOptionInstance o = (lavcOptionInstance)it.next();

            if (o.getLavcOptionString() != null) {
                if (temp == null)
                    temp = o.getLavcOptionString();
                else
                    temp = temp + ":" + o.getLavcOptionString();
            }
        }
        return temp;
    }
}
