/* 
 * Copyright (C) 2004 Romain Dolbeau <romain@dolbeau.org>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

package lavcGenetic;

import java.util.*;

class lavcOption_Integer extends lavcOption {
    private List allInstance = new ArrayList();
    private boolean mandatory;
    private List names;

    public String toString() {
        return("Options: " + allInstance);
    }

    String getDescription() {
        return "Integer: " + names;
    }

    static class lavcOptionInstance_Integer extends lavcOptionInstance {
        int value;
        List names;
        
        lavcOptionInstance_Integer(List n, int i) {
            names = n;
            value = i;
        }
        
        String getLavcOptionString() {
            Iterator it = names.iterator();
            String temp = null;
            while (it.hasNext()) {
                String name = (String)it.next();
                if (temp == null)
                    temp = name + "=" + value;
                else
                    temp = temp + ":" + name + "=" + value;
            }
            return temp;
        }
    }
    
    lavcOption_Integer(String n, int mi, int ma, boolean m) {
        List temp1 = new ArrayList();
        List temp2 = new ArrayList();
        temp1.add(n);
        for (int i = mi ; i <= ma ; i++) {
            temp2.add(new Integer(i));
        }
        init(temp1, temp2, m);
    }

    lavcOption_Integer(String n, List values, boolean m) {
        List temp1 = new ArrayList();
        temp1.add(n);
        init(temp1, values, m);
    }

    lavcOption_Integer(List n, List values, boolean m) {
        init(n, values, m);
    }

    private void init(List n, List values, boolean m) {
        mandatory = m;
        names = new ArrayList(n);

        if (!m) {
            allInstance.add(new lavcOptionInstance() {
                    String getLavcOptionString() {
                        return null;
                    }
                });
        }
        
        Iterator it = values.iterator();
        while (it.hasNext()) {
            Object o = it.next();
            if (o instanceof Integer) {
                allInstance.add(new lavcOptionInstance_Integer(names, ((Integer)o).intValue()));
            } else {
                lavcGeneticMain.err_println("Not Integer: " + o);
            }
        }
    }
    
    
    java.util.List getAllVariants() {
        return Collections.unmodifiableList(allInstance);
    }

}
