/* 
 * Copyright (C) 2004 Romain Dolbeau <romain@dolbeau.org>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

package lavcGenetic;

import java.util.*;

class lavcTestThread extends Thread {
    private lavcEncodeData led;
    private lavcJobDist machines;
    private lavcResultsPool pool;
    private int timeLimit;
    private int index;
    private int bitrate;
    private boolean twopass;

    lavcTestThread(int i, lavcEncodeData l, lavcJobDist m, lavcResultsPool p, int t, int b, boolean tw) {
        led = l;
        machines = m;
        pool = p;
        timeLimit = t;
        index = i;
        bitrate = b;
        twopass = tw;
    }

    public void run() {
        lavcResults r = null, r1 = null, r2 = null;
        String machine = machines.getMachine();
        try {
            lavcMencoderWrapper lmw = new lavcMencoderWrapperRemote(machine);
            if (!twopass) {
                synchronized (led) { led.wait(1000 + 100 * index); } // make sure we don't spawn process too fast
                r = lmw.dumpMoviePassX(led, 0, bitrate, machine);
            } else {
                synchronized (led) { led.wait(1000 + 100 * index); }// make sure we don't spawn process too fast
                r1 = lmw.dumpMoviePassX(led, 1, bitrate, machine);
                if (r1 != null) {
                    synchronized (led) { led.wait(1000 + 100 * index); } // make sure we don't spawn process too fast
                    r2 = lmw.dumpMoviePassX(led, 2, bitrate, machine);
                }
            }
        }
        catch (InterruptedException e) { lavcGeneticMain.err_println("OUPS: " + e); }
        catch (java.rmi.RemoteException e) { lavcGeneticMain.err_println("REMOTE OUPS: " + e); }

        machines.returnMachine(machine);


        if (r != null) {
            pool.addResult(led.lavcOpts, r);
            lavcGeneticMain.err_println("RESULTS ONE PASS (" + index + ", " + machine + "): " + r);
         }
    
        if ((r1 != null) && (r2 != null)) {
            pool.addResult(led.lavcOpts, r2);
            lavcGeneticMain.err_println("RESULTS PASS 1 (" + index + ", " + machine + "): " + r1);
            lavcGeneticMain.err_println("RESULTS PASS 2 (" + index + ", " + machine + "): " + r2);
            lavcGeneticMain.err_println("RESULTS DUAL PASS GAIN (" + index + ", " + machine + "): " + (100.* (r2.psnrAll - r1.psnrAll)/r1.psnrAll) + " %");
        }

        synchronized (machines) {
            machines.notify();
        }
    }
}
